#include <iostream>

#include "team.h"

int main( int argc, char *argv[] )
{
    const char *xml_file = "team.xml";

    if( argc >= 2 )
        xml_file = argv[1];

    // Read the XML data contained in the file 'xml_file' into the
    // project_team object
    lmx::elmx_error error;
    c_ProjectTeam project_team( xml_file, &error );

    // Test whether the data was read successfully (exit if not)
    if( error != lmx::ELMX_OK )
    {
        std::cout << "Unable to unmarshal\n";
        LMX_OUTPUT_DEBUG_ERROR( std::cout );    // Will only output debug info in debug mode
        return 1;
    }

    // Iterate through each team member sub-object and calculate the teams
    // combined brain power
    unsigned int total_brain_power = 0;
    for( size_t i=0; i < project_team.size_Member(); ++i )
        total_brain_power += project_team.get_Member( i ).get_BrainPower();

    // Print out the result
    std::cout <<
            "The total brain power for project team " <<
            lmx::convert_to_narrow( project_team.get_TeamName() ) <<
            " is " << total_brain_power << "\n";

    #ifdef _MSC_VER     // Allow for operation inside IDE
    std::cout << "----Press <Return> to continue----\n";
    std::cin.get();
    #endif

    return 0;
}
